@@ -16,7 +16,7 @@ import time
 import uuid
 from copy import deepcopy
 from datetime import datetime
-from pathlib import PurePosixPath
+from pathlib import Path
 
 import dotenv
 import git
@@ -40,6 +40,7 @@ from calkit.cli.cloud import cloud_app
 from calkit.cli.config import config_app
 from calkit.cli.describe import describe_app
 from calkit.cli.import_ import import_app
+from calkit.cli.latex import latex_app
 from calkit.cli.list import list_app
 from calkit.cli.new import new_app
 from calkit.cli.notebooks import notebooks_app
@@ -70,6 +71,7 @@ app.add_typer(import_app, name="import", help="Import objects.")
 app.add_typer(office_app, name="office", help="Work with Microsoft Office.")
 app.add_typer(update_app, name="update", help="Update objects.")
 app.add_typer(check_app, name="check", help="Check things.")
+app.add_typer(latex_app, name="latex", help="Work with LaTeX.")
 app.add_typer(overleaf_app, name="overleaf", help="Interact with Overleaf.")
 app.add_typer(cloud_app, name="cloud", help="Interact with a Calkit Cloud.")
 app.add_typer(slurm_app, name="slurm", help="Work with SLURM.")
@@ -264,6 +266,11 @@ def get_status():
         calkit.pipeline.to_dvc(ck_info=ck_info, write=True)
     except Exception as e:
         warn(f"Failed to compile pipeline: {e.__class__.__name__}: {e}")
+    # Clean all notebooks in the pipeline
+    try:
+        calkit.notebooks.clean_all_in_pipeline(ck_info=ck_info)
+    except Exception as e:
+        warn(f"Failed to clean notebooks: {e.__class__.__name__}: {e}")
     print_sep("Project")
     # Print latest status
     status = calkit.get_latest_project_status()
@@ -287,7 +294,7 @@ def get_status():
     print_sep("Data (DVC)")
     run_cmd([sys.executable, "-m", "dvc", "data", "status"])
     typer.echo()
-    print_sep("Pipeline (DVC)")
+    print_sep("Pipeline")
     run_cmd([sys.executable, "-m", "dvc", "status"])
 
 
@@ -769,7 +776,7 @@ def ignore(
     """Ignore a file, i.e., keep it out of version control."""
     repo = git.Repo()
     # Ensure path makes it into .gitignore as a POSIX path
-    path = PurePosixPath(path).as_posix()
+    path = Path(path).as_posix()
     if repo.ignored(path):
         typer.echo(f"{path} is already ignored")
         return
@@ -1014,6 +1021,14 @@ def run(
             help="Run stages that produce the given output path.",
         ),
     ] = [],
+    sync_overleaf: Annotated[
+        bool,
+        typer.Option(
+            "--overleaf",
+            "-O",
+            help="Sync with Overleaf before and after running.",
+        ),
+    ] = False,
 ):
     """Check dependencies and run the pipeline."""
     import dvc.log
@@ -1022,6 +1037,10 @@ def run(
     import dvc.ui
     from dvc.cli import main as dvc_cli_main
 
+    import calkit.environments
+    import calkit.pipeline
+    from calkit.cli.overleaf import sync as overleaf_sync
+
     if (target_inputs or target_outputs) and targets:
         raise_error("Cannot specify both targets and inputs")
     os.environ["CALKIT_PIPELINE_RUNNING"] = "1"
@@ -1029,8 +1048,14 @@ def run(
     ck_info = calkit.load_calkit_info()
     # Set env vars
     calkit.set_env_vars(ck_info=ck_info)
+    # Clean all notebooks in the pipeline
+    try:
+        calkit.notebooks.clean_all_in_pipeline(ck_info=ck_info)
+    except Exception as e:
+        raise_error(f"Failed to clean notebooks: {e.__class__.__name__}: {e}")
     if not quiet:
         typer.echo("Getting system information")
+    # Get system information
     system_info = calkit.get_system_info()
     if save_log:
         # Save the system to .calkit/systems
@@ -1047,11 +1072,26 @@ def run(
     if not quiet:
         typer.echo("Checking system-level dependencies")
     try:
-        calkit.check_system_deps(system_info=system_info)
+        calkit.check_system_deps(ck_info=ck_info, system_info=system_info)
     except Exception as e:
         os.environ.pop("CALKIT_PIPELINE_RUNNING", None)
         raise_error(str(e))
-    # Compile the pipeline
+    # Check all environments in the pipeline (with caching)
+    # If any failed, warn the user that we might have problems running
+    typer.echo("Checking environments")
+    env_check_results = calkit.environments.check_all_in_pipeline(
+        ck_info=ck_info, targets=targets, force=force
+    )
+    for env_name, result in env_check_results.items():
+        if verbose:
+            typer.echo(f"{env_name}: {result}")
+        failed = not result.get("success", False)
+        if failed:
+            warn(f"Failed to check environment '{env_name}'")
+    # If specified, perform initial Overleaf sync
+    if sync_overleaf:
+        overleaf_sync(no_commit=False, no_push=True, verbose=verbose)
+    # Compile the DVC pipeline
     dvc_stages = None
     if ck_info.get("pipeline", {}):
         if not quiet:
@@ -1236,6 +1276,9 @@ def run(
                 "utf-8", errors="replace"
             )
         )
+    # If specified, perform final Overleaf sync
+    if sync_overleaf:
+        overleaf_sync(verbose=verbose, no_commit=save_after_run, no_push=True)
     if save_after_run or save_message is not None:
         if save_message is None:
             save_message = "Run pipeline"
@@ -2013,49 +2056,117 @@ def run_jupyter(
     sys.exit(process.returncode)
 
 
-@app.command(name="latexmk")
-def run_latexmk(
-    tex_file: Annotated[str, typer.Argument(help="The .tex file to compile.")],
-    environment: Annotated[
-        str | None,
+@app.command(name="map-paths")
+def map_paths(
+    file_to_file: Annotated[
+        list[str],
         typer.Option(
-            "--env",
-            "-e",
-            help=("Environment in which to run latexmk, if applicable."),
+            "--file-to-file",
+            help=(
+                "Map a file to another file, e.g., "
+                "--file-to-file 'results.tex->paper/results.tex'."
+            ),
         ),
-    ] = None,
+    ] = [],
+    file_to_dir: Annotated[
+        list[str],
+        typer.Option(
+            "--file-to-dir",
+            help=(
+                "Map a file into a directory, e.g., "
+                "--file-to-dir 'results.tex->paper/results'."
+            ),
+        ),
+    ] = [],
+    dir_to_dir_replace: Annotated[
+        list[str],
+        typer.Option(
+            "--dir-to-dir-replace",
+            help=(
+                "Copy directory to another directory and replace it, "
+                "e.g., --dir-to-dir-replace 'figures->paper/figures'."
+            ),
+        ),
+    ] = [],
+    dir_to_dir_merge: Annotated[
+        list[str],
+        typer.Option(
+            "--dir-to-dir-merge",
+            help=(
+                "Merge directory into another directory. "
+                "This is useful for merging contents of one directory into "
+                "another, e.g., --dir-to-dir-merge 'figures->paper/figures'."
+            ),
+        ),
+    ] = [],
 ):
-    """Compile a LaTeX document with latexmk.
+    """Map paths in a project.
 
-    If a Calkit environment is not specified, latexmk will be run in the
-    system environment if available. If not available, a TeX Live Docker
-    container will be used.
+    Currently this is done with copying. Outputs are ensured to be ignored by
+    Git.
     """
-    latexmk_cmd = [
-        "latexmk",
-        "-pdf",
-        "-cd",
-        "-silent",
-        "-synctex=1",
-        "-interaction=nonstopmode",
-        tex_file,
-    ]
-    if environment is not None:
-        cmd = ["calkit", "xenv", "--name", environment] + latexmk_cmd
-    elif calkit.check_dep_exists("latexmk"):
-        cmd = latexmk_cmd
-    else:
-        cmd = [
-            "docker",
-            "run",
-            "--rm",
-            "-v",
-            f"{os.getcwd()}:/work",
-            "-w",
-            "/work",
-            "texlive/texlive:latest-full",
-        ] + latexmk_cmd
-    try:
-        subprocess.check_call(cmd)
-    except subprocess.CalledProcessError:
-        raise_error("latexmk failed")
+    repo = git.Repo()
+
+    def validate_and_split(mapping: str) -> tuple[str, str]:
+        if "->" not in mapping:
+            raise_error(
+                f"Invalid path mapping format: '{mapping}'; "
+                "Expected format: 'src->dest'"
+            )
+        parts = mapping.split("->")
+        if len(parts) != 2:
+            raise_error(
+                f"Invalid path mapping format: '{mapping}'; "
+                "Expected exactly one '->' separator"
+            )
+        return parts[0].strip(), parts[1].strip()
+
+    for copy_file in file_to_file:
+        src_path, dest_path = validate_and_split(copy_file)
+        if os.path.isdir(dest_path):
+            raise_error(f"Destination path '{dest_path}' is a directory")
+        parent_dir = os.path.dirname(dest_path)
+        if parent_dir:
+            os.makedirs(parent_dir, exist_ok=True)
+        shutil.copy2(src_path, dest_path)
+        calkit.git.ensure_path_is_ignored(repo, path=dest_path)
+    for copy_file in file_to_dir:
+        src_path, dest_dir = validate_and_split(copy_file)
+        if os.path.isfile(dest_dir):
+            raise_error(f"Destination path '{dest_dir}' is a file")
+        if not os.path.isdir(dest_dir):
+            os.makedirs(dest_dir, exist_ok=True)
+        dest_path = os.path.join(dest_dir, os.path.basename(src_path))
+        shutil.copy2(src_path, dest_path)
+        calkit.git.ensure_path_is_ignored(repo, path=dest_path)
+    for replace_dir_with_dir in dir_to_dir_replace:
+        src_dir, dest_dir = validate_and_split(replace_dir_with_dir)
+        if os.path.isfile(dest_dir):
+            raise_error(f"Destination path '{dest_dir}' is a file")
+        if os.path.isfile(src_dir):
+            raise_error(f"Source path '{src_dir}' is a file")
+        if os.path.isdir(dest_dir):
+            shutil.rmtree(dest_dir)
+        parent_dir = os.path.dirname(dest_dir)
+        if parent_dir:
+            os.makedirs(parent_dir, exist_ok=True)
+        shutil.copytree(src_dir, dest_dir)
+        calkit.git.ensure_path_is_ignored(repo, path=dest_dir)
+    for merge_dir_to_dir in dir_to_dir_merge:
+        src_dir, dest_dir = validate_and_split(merge_dir_to_dir)
+        if os.path.isfile(dest_dir):
+            raise_error(f"Destination path '{dest_dir}' is a file")
+        if os.path.isfile(src_dir):
+            raise_error(f"Source path '{src_dir}' is a file")
+        if not os.path.isdir(dest_dir):
+            os.makedirs(dest_dir, exist_ok=True)
+        for item in os.listdir(src_dir):
+            if item.startswith("."):
+                continue
+            src_item = os.path.join(src_dir, item)
+            dest_item = os.path.join(dest_dir, item)
+            if os.path.isdir(src_item):
+                shutil.copytree(src_item, dest_item, dirs_exist_ok=True)
+            else:
+                shutil.copy2(src_item, dest_item)
+            calkit.git.ensure_path_is_ignored(repo, path=dest_item)
